@@ -1,26 +1,33 @@
 use kwap_common::Array;
-use kwap_msg::{EnumerateOptNumbers, Message, Opt, OptNumber, Payload, Token, TryIntoBytes, Type};
-#[cfg(feature = "alloc")]
-use std_alloc::string::{FromUtf8Error, String};
+use kwap_msg::{EnumerateOptNumbers, Message, Opt, OptNumber, Payload, TryIntoBytes, Type};
 
-#[doc(hidden)]
+use crate::ToCoapValue;
+
+/// Request methods
 pub mod method;
+
 #[doc(inline)]
 pub use method::Method;
 
+/// Request builder
+pub mod builder;
+
+#[doc(inline)]
+pub use builder::*;
+
 use crate::config::{self, Config};
 
 /// A CoAP request
 ///
 /// ```
-/// use kwap::config::Alloc;
+/// use kwap::config::Std;
 /// use kwap::req::Req;
 /// use kwap::resp::Resp;
 ///
 /// # main();
 /// fn main() {
 ///   let client = Client::new();
-///   let mut req = Req::<Alloc>::post("coap://myfunnyserver.com", 5632, "hello");
+///   let mut req = Req::<Std>::post("coap://myfunnyserver.com", 5632, "hello");
 ///   req.set_payload("john".bytes());
 ///
 ///   let resp = client.send(req);
@@ -39,7 +46,7 @@ use crate::config::{self, Config};
 ///     # Self {__field: ()}
 ///   }
 ///
-///   fn send(&self, req: Req<Alloc>) -> Resp<Alloc> {
+///   fn send(&self, req: Req<Std>) -> Resp<Std> {
 ///     // send the request
 ///     # let body = req.payload_str().unwrap().to_string();
 ///     # let mut resp = Resp::for_request(req);
@@ -51,7 +58,7 @@ use crate::config::{self, Config};
 #[derive(Debug, Clone)]
 pub struct Req<Cfg: Config> {
   pub(crate) msg: config::Message<Cfg>,
-  opts: Option<Cfg::OptNumbers>,
+  opts: Option<Cfg::OptMap>,
 }
 
 impl<Cfg: Config> Req<Cfg> {
@@ -106,10 +113,10 @@ impl<Cfg: Config> Req<Cfg> {
   /// Get a copy of the message id for this request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let req = Req::<Alloc>::get("1.1.1.1", 5683, "/hello");
+  /// let req = Req::<Std>::get("1.1.1.1", 5683, "/hello");
   /// let _msg_id = req.msg_id();
   /// ```
   pub fn msg_id(&self) -> kwap_msg::Id {
@@ -127,26 +134,36 @@ impl<Cfg: Config> Req<Cfg> {
   /// Otherwise, returns `None`.
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let mut req = Req::<Alloc>::get("1.1.1.1", 5683, "/hello");
+  /// let mut req = Req::<Std>::get("1.1.1.1", 5683, "/hello");
   /// req.set_option(17, Some(50)); // Accept: application/json
   /// ```
   pub fn set_option<V: IntoIterator<Item = u8>>(&mut self, number: u32, value: V) -> Option<(u32, V)> {
     if self.opts.is_none() {
       self.opts = Some(Default::default());
     }
-    crate::add_option(self.opts.as_mut().unwrap(), number, value)
+
+    crate::option::add(self.opts.as_mut().unwrap(), false, number, value)
+  }
+
+  /// Add an instance of a repeatable option to the request.
+  pub fn add_option<V: IntoIterator<Item = u8>>(&mut self, number: u32, value: V) -> Option<(u32, V)> {
+    if self.opts.is_none() {
+      self.opts = Some(Default::default());
+    }
+
+    crate::option::add(self.opts.as_mut().unwrap(), true, number, value)
   }
 
   /// Creates a new GET request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let _req = Req::<Alloc>::get("1.1.1.1", 5683, "/hello");
+  /// let _req = Req::<Std>::get("1.1.1.1", 5683, "/hello");
   /// ```
   pub fn get(host: impl AsRef<str>, port: u16, path: impl AsRef<str>) -> Self {
     Self::new(Method::GET, host, port, path)
@@ -155,10 +172,10 @@ impl<Cfg: Config> Req<Cfg> {
   /// Creates a new POST request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let mut req = Req::<Alloc>::post("1.1.1.1", 5683, "/hello");
+  /// let mut req = Req::<Std>::post("1.1.1.1", 5683, "/hello");
   /// req.set_payload("Hi!".bytes());
   /// ```
   pub fn post(host: impl AsRef<str>, port: u16, path: impl AsRef<str>) -> Self {
@@ -168,10 +185,10 @@ impl<Cfg: Config> Req<Cfg> {
   /// Creates a new PUT request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let mut req = Req::<Alloc>::put("1.1.1.1", 5683, "/hello");
+  /// let mut req = Req::<Std>::put("1.1.1.1", 5683, "/hello");
   /// req.set_payload("Hi!".bytes());
   /// ```
   pub fn put(host: impl AsRef<str>, port: u16, path: impl AsRef<str>) -> Self {
@@ -181,10 +198,10 @@ impl<Cfg: Config> Req<Cfg> {
   /// Creates a new DELETE request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let _req = Req::<Alloc>::delete("1.1.1.1", 5683, "/users/john");
+  /// let _req = Req::<Std>::delete("1.1.1.1", 5683, "/users/john");
   /// ```
   pub fn delete(host: impl AsRef<str>, port: u16, path: impl AsRef<str>) -> Self {
     Self::new(Method::DELETE, host, port, path)
@@ -193,23 +210,23 @@ impl<Cfg: Config> Req<Cfg> {
   /// Add a payload to this request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let mut req = Req::<Alloc>::put("1.1.1.1", 5683, "/hello");
+  /// let mut req = Req::<Std>::put("1.1.1.1", 5683, "/hello");
   /// req.set_payload("Hi!".bytes());
   /// ```
-  pub fn set_payload<P: IntoIterator<Item = u8>>(&mut self, payload: P) {
-    self.msg.payload = Payload(payload.into_iter().collect());
+  pub fn set_payload<P: ToCoapValue>(&mut self, payload: P) {
+    self.msg.payload = Payload(payload.to_coap_value::<Cfg::PayloadBuffer>());
   }
 
   /// Get the payload's raw bytes
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let mut req = Req::<Alloc>::post("1.1.1.1", 5683, "/hello");
+  /// let mut req = Req::<Std>::post("1.1.1.1", 5683, "/hello");
   /// req.set_payload("Hi!".bytes());
   ///
   /// assert!(req.payload().iter().copied().eq("Hi!".bytes()))
@@ -221,10 +238,10 @@ impl<Cfg: Config> Req<Cfg> {
   /// Read an option by its number from the request
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let req = Req::<Alloc>::post("1.1.1.1", 5683, "/hello");
+  /// let req = Req::<Std>::post("1.1.1.1", 5683, "/hello");
   /// let uri_host = req.get_option(3).unwrap();
   /// assert_eq!(uri_host.value.0, "1.1.1.1".bytes().collect::<Vec<_>>());
   /// ```
@@ -237,22 +254,22 @@ impl<Cfg: Config> Req<Cfg> {
   /// Get the payload and attempt to interpret it as an ASCII string
   ///
   /// ```
-  /// use kwap::config::Alloc;
+  /// use kwap::config::Std;
   /// use kwap::req::Req;
   ///
-  /// let mut req = Req::<Alloc>::post("1.1.1.1", 5683, "/hello");
+  /// let mut req = Req::<Std>::post("1.1.1.1", 5683, "/hello");
   /// req.set_payload("Hi!".bytes());
   ///
   /// assert_eq!(req.payload_str().unwrap(), "Hi!")
   /// ```
   pub fn payload_str(&self) -> Result<&str, core::str::Utf8Error> {
-    core::str::from_utf8(&self.payload())
+    core::str::from_utf8(self.payload())
   }
 
   /// Drains the internal associated list of opt number <> opt and converts the numbers into deltas to prepare for message transmission
   fn normalize_opts(&mut self) {
     if let Some(opts) = Option::take(&mut self.opts) {
-      self.msg.opts = crate::normalize_opts(opts);
+      self.msg.opts = crate::option::normalize(opts);
     }
   }
 
